@@ -1,4 +1,4 @@
-use serde::*;
+use serde::Deserialize;
 
 #[derive(Deserialize, Debug, Clone)]
 pub struct SearchResponse {
